<?php
/**
 * @package    JMB Comments
 * @author     Sherza (sherza.web@gmail.com)
 * @copyright  (c) 2012 - 2013 Joomlablog.ruю All rights reserved.
 * @license    GNU General Public License version 3 or later; see license.txt
 */

defined('JPATH_BASE') or die;

/**
 * Dropdown field class.
 *
 * @package  JMB Comments
 */
class JFormFieldDropdown extends JFormField
{
	/**
	 * Field name.
	 *
	 * @var  string
	 */
	protected $type = 'dropdown';

	/**
	 * Method to get field input.
	 *
	 * @return  string  HTML output.
	 */
	protected function getInput()
	{
		$doc = JFactory::getDocument();
		$doc->addScript(JURI::root() . 'plugins/content/jmb_comments/fields/drag.js');
		$doc->addStyleSheet(JURI::root() . 'plugins/content/jmb_comments/fields/drag.css');

		$values = array_map('trim', explode(",", $this->value));

		$lis = array(); 
		$lis[0] = array(); 
		$lis[1] = array();

		$options = (array)$this->getOptions();

		$allValues = array();
		foreach($options as $option)
		{
			$used = (in_array($option->value, $values)) ? 1 : 0;
			$lis[$used][$option->value] = '<li class="zeLi" fid="' . $this->id . '" rel="' . $option->value . '"><span class="zeFieldLabel">' . $option->text . '</span><span onclick="zeOpenClose(this)" class="zeOpenCloseSpan" id="zeOpenCloseSpan_' . $this->id . '_' . $option->value . '"></span><div class="ze_fieldtype_alias_div" id="ze_field_' . $this->id . '_' . $option->value . '"></div></li>';
			$allValues[] = $option->value;
		}

		$script = '
		var zeOptsArray=["' . implode('","', $allValues) . '"];
		window.addEvent("domready", function() { 
			list1 = document.getElementById("ze_' . $this->id . '_available_fields");
			if (list1) {
				ZDragDrop.makeListContainer( list1 );
				list1.onZDragOver = function() { this.style["border"] = "1px dashed blue"; this.style["background"] = "#FFF"; };
				list1.onZDragOut = function() {this.style["border"] = "1px solid transparent"; this.style["background"] = "none";};
			}

			list2 = document.getElementById("ze_' . $this->id . '_fields_in_use");
			if( list2) {
				ZDragDrop.makeListContainer( list2 );
				list2.onZDragOver = function() { this.style["border"] = "1px dashed #8E0B8C"; this.style["background"] = "#FFF"; };
				list2.onZDragOut = function() {this.style["border"] = "1px solid transparent"; this.style["background"] = "none";};
			}

			zeOptsArray.each(function(el, i){
				if ($(el+"-set-options")) {
					$("ze_field_' . $this->id . '_"+el).appendChild($(el+"-set-options").getParent().getChildren("div.pane-slider")[0].getChildren("fieldset")[0]);
					$(el+"-set-options").getParent().destroy();
				} else {
					$("zeOpenCloseSpan_' . $this->id . '_"+el).style.display="none";
				}
			});

			$("zeDropdownFields_' . $this->id . '").getParent().getParent().addClass("zeUladminformlist");
			var wrapperLis=$("zeDropdownFields_' . $this->id . '").getParent().getParent().getChildren("li");
			wrapperLis.each(function(el, i){
				if(i){
					oddEven=(i%2)? "odd" : "even";
					el.addClass("zeLiWrap "+oddEven);
					var zdiv=document.createElement("div");
					zdiv.className="zeClear";
					el.appendChild(zdiv);
				}
			});
		'; 

		$script .= '});';
		$script .= '
			function zeOpenClose(self) {
				self.parentNode.className= (self.parentNode.className == "zeOpened") ? "" : "zeOpened";
			}';

		$doc->addScriptDeclaration($script);
?>
		<input type="hidden" name="<?php echo $this->name; ?>" id="<?php echo $this->id; ?>" value="<?php echo $this->value; ?>">
		<div style="clear:both"></div>
		<div class="zeDropdownFields" id="zeDropdownFields_<?php echo $this->id; ?>" >
			<fieldset class="boxierFiedset">
				<legend><?php echo JText::_('PLG_CONTENT_JMB_COMMENTS_USED'); ?></legend>
				<ul id="ze_<?php echo $this->id?>_fields_in_use" class="sortable boxier">
					<?php 
						foreach($values as $value){
							echo $lis[1][$value];
						}
					?>
				</ul>
			</fieldset>
			<fieldset class="boxyFiedset"><legend><?php echo JText::_('PLG_CONTENT_JMB_COMMENTS_AVAILABLE'); ?></legend>
				<ul id="ze_<?php echo $this->id?>_available_fields" class="sortable boxy">
					<?php echo implode('', $lis[0]);?>
				</ul>
		   </fieldset>
		</div>
<?php
	}

	/**
	 * Method to get field options.
	 *
	 * @return  string  HTML output.
	 */
	protected function getOptions()
	{
		// Initialize variables.
		$options = array();

		foreach ($this->element->children() as $option)
		{
			// Only add <option /> elements.
			if ($option->getName() != 'option')
			{
				continue;
			}

			// Create a new option object based on the <option /> element.
			$tmp = JHtml::_(
				'select.option', (string) $option['value'],
				JText::alt(trim((string) $option), preg_replace('/[^a-zA-Z0-9_\-]/', '_', $this->fieldname)), 'value', 'text',
				((string) $option['disabled'] == 'true')
			);

			// Set some option attributes.
			$tmp->class = (string) $option['class'];

			// Set some JavaScript option attributes.
			$tmp->onclick = (string) $option['onclick'];

			// Add the option object to the result set.
			$options[] = $tmp;
		}

		reset($options);

		return $options;
	}
}
