<?php
/**
 * @package    JMB Comments
 * @author     Sherza (sherza.web@gmail.com)
 * @copyright  (c) 2012 - 2013 Joomlablog.ruю All rights reserved.
 * @license    GNU General Public License version 3 or later; see license.txt
 */

defined('_JEXEC') or die;

/**
 * JMB Comments plugin base class.
 *
 * @package  JMB Comments
 */
class plgContentJMB_comments extends JPlugin
{
	/**
	 * Constructor.
	 *
	 * @param  object  $subject  The object to observe.
	 * @param  array   $config   An array that holds the plugin configuration.
	 *
	 */
	public function __construct(&$subject, $params)
	{
		parent::__construct($subject, $params);
		$this->loadLanguage();
	}

	/**
	 * Method to add comments form based on onContentAfterDisplay event trigger.
	 *
	 * @param   string  $context  The context of the content being passed to the plugin.
	 * @param   mixed   $row      An object with a "text" property.
	 * @param   array   $params   Additional parameters.
	 * @param   int     $page     Optional page number. Unused. Defaults to zero.
	 *
	 * @return  string  Modified text.
	 */
	public function onContentAfterDisplay($context, &$row, &$params, $page = 0)
	{
		if ($context != 'com_content.article') 
		{
			return;
		}

		if (!$this->params->get('comments', 'ko,jc,vk')) 
		{
			return;
		}

		$show_cats = $this->params->get('show_cats');
		$cats = $this->params->get('cats');
		$show_articles = $this->params->get('show_articles');
		$articles = $this->params->get('articles');

		if (!($show_articles=='in' && !empty($articles) && in_array($row->id, $articles)))
		{
			if ($show_articles=='ex' && !empty($articles) && in_array($row->id, $articles))
			{
				return;
			}
			elseif ($show_articles=='in' 
					&& !empty($articles) 
					&& !in_array($row->id, $articles) 
					&& !($show_cats=='in' 
					&& !empty($cats) 
					&& in_array($row->catid, $cats))
					)
			{
				return;
			}
			elseif ($show_cats=='ex' && !empty($cats) && in_array($row->catid, $cats))
			{
				return;
			}
		}

		$values = array_map('trim', explode(",", $this->params->get('comments', 'ko,jc,vk')));
		$comments = array();
		foreach($values as $value)
		{
			$comment = $this->$value($row, $page);
			if ($comment) 
			{
				$comments[$value] = $comment;
			}
		}
		if (empty($comments)) 
		{
			return;
		}

		$doc = JFactory::getDocument();
		$doc->addStyleSheet(JURI::root().'plugins/content/jmb_comments/jmb_comments.css');

		$texticon = $this->params->get('texticon', 'texticon');

		$html .= '<div class="zeJMB_commentsWrapper">';
			$html .= '<div class="zeJMB_commentsTabs ' . $texticon . '">';
			$k = 0;
			foreach($comments as $value => $comment)
			{
				$active = (!$k) ? ' active ' : '';
				$html .= '<span class="zeJMB_commentsTab zeJMB_commentsTab' . $value . $active.'" onclick="zeShowComment(this, \'' . $value . '\')">';
				if ($texticon=='texticon' || $texticon == 'icon') 
				{
					$html .= '<img src="' . JURI::base() . 'plugins/content/jmb_comments/images/' . $value . '-16.png" />';
				}

				if ($texticon=='texticon' || $texticon == 'text') 
				{
					$html .= '<span>' . JText::_('PLG_CONTENT_JMB_COMMENTS_' . strtoupper($value) . '_NAME_OPTION') . '</span>';
				}
				$html .= '</span>';
				$k++;
			}
			$html .= '</div>';
			$html .= '<div class="zeJMB_commentsContents">';
			$k = 0;
			foreach ($comments as $value => $comment)
			{
				$active = (!$k) ? ' active ' : '';
				$html .= '<span class="zeJMB_commentsContent zeJMB_commentsContent' . $value . $active . '">';
				$html .= $comment;
				$html .= '</span>';
				$k++;
			}
			$html .= '</div>';
		$html .= '</div>';
		$html .= '<script type="text/javascript">';
		$html .= 'function zeShowComment(self, comment){
				otherTabs=self.parentNode.childNodes;
				for(i=0; i<otherTabs.length; i++){
					otherTabs[i].className = otherTabs[i].className.replace("active", "");
				}
				self.className=self.className.replace("active", "")+" active";

				wrapper=self.parentNode.parentNode.childNodes;
				for(i=0; i<wrapper.length; i++){
					if(wrapper[i].className == "zeJMB_commentsContents") contentWrapper=wrapper[i];
				}
				for(i=0; i<contentWrapper.childNodes.length; i++){
					cnt=contentWrapper.childNodes[i];
					isActive=(cnt.className.replace("zeJMB_commentsContent"+comment, "")!=cnt.className)? " active" : "";
					cnt.className = cnt.className.replace("active", "")+isActive;
				}
				
			}';
		$html .= '</script>';
		$html .= '
			<div style="text-align: right">
				<a href="http://joomlablog.ru/" target="_blank" style="font-family: arial,helvetica,sans-serif; font-size: 5pt;text-decoration:none;color:#bfbfbf;">Расширения Joomla</a>
			</div>
			';

		$row->text .= $html;
	}

	/**
	 * Method to get VK comments form.
	 *
	 * @param   mixed   $article  An object with a "text" property.
	 * @param   int     $page     Optional page number.
	 *
	 * @return  string  Modified text.
	 */
	function vk(&$article, $page)
	{
		$doc = JFactory::getDocument();

		$apiId = $this->params->def('apiId');
		$width = $this->params->def('width');
		$comLimit = $this->params->def('comLimit');
		$attach = $this->params->def('attach');
		$autoPublish = $this->params->def('autoPublish');
		$norealtime = $this->params->def('norealtime');
		$script = "VK.init({apiId: $apiId, onlyWidgets: true});";

		$doc->addScript("http://vkontakte.ru/js/api/openapi.js");
		$doc->addScriptDeclaration($script);

		$pagehash = $article->id;
		$vkcomments = <<<HTML
		
			<div id='jlvkcomments'></div>
			<script type='text/javascript'>
			VK.Widgets.Comments('jlvkcomments', {limit: $comLimit, width: '$width', attach: '$attach', autoPublish: $autoPublish, norealtime: $norealtime},$pagehash);
			</script>
		
HTML;
		return $vkcomments;
	}

	/**
	 * Method to get Facebook comments form.
	 *
	 * @param   mixed   $article  An object with a "text" property.
	 * @param   int     $page     Optional page number.
	 *
	 * @return  string  Modified text.
	 */
	function fb(&$article, $page)
	{
		require_once(JPATH_BASE.DS . 'components' . DS . 'com_content' . DS . 'helpers' . DS . 'route.php');

		if ($article->id)
		{
			$link = JRoute::_(ContentHelperRoute::getArticleRoute($article->id, $article->catid));
			$jURI = JURI::getInstance();
			$link = $jURI->getScheme()."://" . $jURI->getHost() . $link;
		}
		else
		{
			$jURI = JURI::getInstance();
			$link = $jURI->toString();
		}

		$html='';
		$doc = JFactory::getDocument();
		$config = JFactory::getConfig();
		$fb_id = $this->params->def('fb_id');
		$fbcomments_width = $this->params->def('fbcomments_width');
		$fb_num = $this->params->def('fb_num');
		$fb_admin_id = $this->params->def('fb_admin_id');
		$fbcomments_color = $this->params->def('fbcomments_color');
		$fbcomments_lang = $this->params->def('fbcomments_lang');

		if ($this->params->get('fb_show_og_tag') == 1)
		{
			$doc->addCustomTag('<meta property="fb:admins" content="' . $fb_admin_id . '"/>');
			$doc->addCustomTag('<meta property="fb:app_id" content="' . $fb_id . '"/>');
			$doc->addCustomTag('<meta property="og:title" content="' . $article->title . '"/>');
			$doc->addCustomTag('<meta property="og:type" content="article"/>');
			$doc->addCustomTag('<meta property="og:url" content="' . $link . '"/>');
			$doc->addCustomTag('<meta property="og:site_name" content="' . $config->getValue('sitename') . '"/>');
		    $pattern = "/<img[^>]*src\=['\"]?(([^>]*)(jpg|gif|png|jpeg))['\"]?/";
			preg_match($pattern, $article->text, $matches);
			if (!empty($matches))
			{
				$doc->addCustomTag('<meta property="og:image" content="' . $root . '/' . $matches[1] . '"/>');
			}
		}
		$html .= '<div  id="tabs-1">';
		$html .= '<div id="fb-root"></div>';
		$html .= '<script src="http://connect.facebook.net/' . $fbcomments_lang . '/all.js#xfbml=1"></script><fb:comments href="' . $link . '" num_posts="' .$fb_num . '" width="' . $fbcomments_width . '" colorscheme="' . $fbcomments_color . '">';
		$html .= '</fb:comments>';
		$html .= '</div>';

		return $html;
	}

	/**
	 * Method to get JComments comments form.
	 *
	 * @param   mixed   $article  An object with a "text" property.
	 * @param   int     $page     Optional page number.
	 *
	 * @return  string  Modified text.
	 */
	function jc(&$article, $page)
	{
		$comments = JPATH_BASE . '/components/com_jcomments/jcomments.php';
		if (file_exists($comments)) 
		{
			require_once($comments);
			$html = '';
			$html .=  JComments::show($article->id, 'com_content', $article->title);
			return $html;
		}
		else
		{
			return;
		}
	}

	/**
	 * Method to get Komento comments form.
	 *
	 * @param   mixed   $article  An object with a "text" property.
	 * @param   int     $page     Optional page number.
	 *
	 * @return  string  Modified text.
	 */
	function ko(&$article, $page)
	{
		$comments = JPATH_ROOT . DS . 'components' . DS . 'com_komento' . DS . 'bootstrap.php';

		if (file_exists($comments)) 
		{
			require_once($comments);
			$html = '';

			$options['context']	= 'com_content.article';
			$options['params']	= $this->params;
			$options['page']	= $page;

			// Ready to Commentify!
			$html .= Komento::commentify( 'com_content', $article, $options );
			return $html;
		}
		else
		{
			return;
		}
	}

	/**
	 * Method to get Disqus comments form.
	 *
	 * @param   mixed   $article  An object with a "text" property.
	 * @param   int     $page     Optional page number.
	 *
	 * @return  string  Modified text.
	 */
	function ds(&$article, $page)
	{
		$disqus_subdomain = $this->params->def('disqus_subdomain');
		$disqus_lang = $this->params->def('disqus_lang');
		$html = '';
		$html .= '<div id="disqus_thread"></div><script type="text/javascript">var disqus_shortname = \'' . $disqus_subdomain . '\';
		var disqus_config = function () {
			this.language = "' . $disqus_lang . '";
		};
		(function() {
		var dsq = document.createElement(\'script\'); dsq.type = \'text/javascript\'; dsq.async = true;
		dsq.src = \'http:\/\/\' + disqus_shortname + \'.disqus.com\/embed.js\';
		(document.getElementsByTagName(\'head\')[0] || document.getElementsByTagName(\'body\')[0]).appendChild(dsq);})();
		</script>
		<noscript>Please enable JavaScript to view the <a href="http://disqus.com/?ref_noscript">comments powered by Disqus.</a></noscript>
		<a href="http://disqus.com" class="dsq-brlink" rel="nofollow">blog comments powered by <span class="logo-disqus">Disqus</span></a>';

		return $html;
	}
}
